/*
 ATtiny85 SPI
 Version: 1.0
 Author: Alex from Inside Gadgets (http://www.insidegadgets.com)
 Created: 6/05/2012
 Last Modified: 6/05/2012
 
 Test on an external SRAM with SPI using USI
 
 */

#define F_CPU 4000000 // 4 MHz

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <math.h>
#include <util/delay.h>
#include "setup.c"

// ATtiny45/85 Pin map
//                        +-\/-+
// Reset/Ain0 (D 5) PB5  1|o   |8  Vcc
//       Ain3 (D 3) PB3  2|    |7  PB2 (D 2) Ain1
//       Ain2 (D 4) PB4  3|    |6  PB1 (D 1) pwm1
//                  GND  4|    |5  PB0 (D 0) pwm0
//                        +----+

#define ledPin PB3

uint8_t spi_transfer(uint8_t data) {
	USIDR = data;
	USISR = _BV(USIOIF); // clear flag

	while ( (USISR & _BV(USIOIF)) == 0 ) {
		USICR = (1<<USIWM0)|(1<<USICS1)|(1<<USICLK)|(1<<USITC);
	}
	return USIDR;
}

int main(void) {
	
	setup();
	
	DDRB |= (1<<ledPin); // SPI C1S (LED connected here too)
	DDRB |= (1<<PB1); // SPI DO
	DDRB |= (1<<PB2); // SPI CLK
	DDRB &= ~(1<<PB0); // SPI DI
	PORTB |= (1<<PB0); // SPI DI
	
	while(1) {
		
		PORTB |= (1<<ledPin); // CS High, in-active
		_delay_ms(1000);
		
		// Enable sequential mode by writing to the register
		PORTB &= ~(1<<ledPin); // CS Low, active
		spi_transfer(0x01);
		spi_transfer(0x40);
		PORTB |= (1<<ledPin); // CS High, in-active
		_delay_ms(10);
		
		// Write 123 and 213 to first and second address of SRAM
		PORTB &= ~(1<<ledPin); // CS Low, active
		spi_transfer(0x02);
		spi_transfer(0);
		spi_transfer(0);
		spi_transfer(123);
		spi_transfer(213);
		PORTB |= (1<<ledPin); // CS High, in-active
		_delay_ms(10);
		
		// Read first and second address of SRAM
		PORTB &= ~(1<<ledPin); // CS Low, active
		spi_transfer(0x03);
		spi_transfer(0);
		spi_transfer(0);
		int firstaddress = spi_transfer(0);
		int secondaddress = spi_transfer(0);
		PORTB |= (1<<ledPin); // CS High, in-active
		
		// If first address reads 123 then blink the LED
		if (firstaddress == 123 && secondaddress == 213) {
			int x = 0;
			for (x = 0; x < 20; x++) {
				PORTB |= (1<<ledPin);
				_delay_ms(200);
				PORTB &= ~(1<<ledPin);
				_delay_ms(200);
			}	
		}	
	}
}
